/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/  

#include "cebomsrpp.h"

#include <stdexcept>
#include <algorithm>
#include <cstring>
#include <iostream>

using namespace std;

namespace CeboMsr {

/**
 * Checks return code of CeboMsr* C functions. If an error is indicated,
 * the error message is read from the low level API and thrown as matching form of std::exception.
 */
void _checkError(const _c::CeboMsrErrorIndex &idx) {
	// Error occurred ?
	if (0 != idx) {
		// Yes, read string length of error message.
		int len = _c::CeboMsrGetErrorLength(idx);

		// Read the error string.
		vector<char> tmp(len + 1, 0);
		_c::CeboMsrGetError(idx, &tmp[0]);

		// The highest nibble specifies the error class.
		switch (idx & 0xf0000000) {
		case 0x80000000: throw std::invalid_argument(&tmp[0]);
		case 0x40000000: throw std::out_of_range(&tmp[0]);
		case 0x20000000: throw std::logic_error(&tmp[0]);
		case 0x10000000: throw std::runtime_error(&tmp[0]);
		}

		// Throw as exception.
		cerr << "Internal error, this shouldn't be the case: " << &tmp[0] << endl;
		throw std::bad_exception();
	}
}

DeviceType DeviceType::All(_c::CeboMsrTypeAll, "All");
DeviceType DeviceType::Usb(_c::CeboMsrTypeUsb, "Usb");
DeviceType DeviceType::CeboLC(_c::CeboMsrTypeCeboLC, "CeboLC");
DeviceType DeviceType::CeboStick(_c::CeboMsrTypeCeboStick, "CeboStick");
DeviceType DeviceType::CeboDFN(_c::CeboMsrTypeCeboDFN, "CeboDFN");

DeviceType &DeviceType::getTypeById(_c::CeboMsrType msrType) {
	switch (msrType) {
	case _c::CeboMsrTypeAll: return All;
	case _c::CeboMsrTypeUsb: return Usb;
	case _c::CeboMsrTypeCeboLC: return CeboLC;
	case _c::CeboMsrTypeCeboStick: return CeboStick;
	case _c::CeboMsrTypeCeboDFN: return CeboDFN;
	}
	throw std::out_of_range("Invalid id.");
}

AnalogInput::AnalogInput(_c::CeboMsrHandle handle, const _c::CeboMsrIo &io, Style style, int styleIndex) :
	Input(io), _handle(handle), _style(style), _styleIndex(styleIndex) {
	for (int i = 0;; ++i) {
		_c::CeboMsrRange rangeId;
		_checkError(_c::CeboMsrEnumerateRanges(_handle, _inputIo, i, &rangeId));
		if (rangeId == _c::CeboMsrRangeInvalid)
			break;
		float minValue, maxValue;
		_checkError(_c::CeboMsrGetRangeMinMax(_handle, _inputIo, rangeId, &minValue, &maxValue));
		int defaultInterChannelDelay;
		_checkError(_c::CeboMsrGetDefaultAnalogInputInterChannelDelay(_handle, _inputIo, rangeId, &defaultInterChannelDelay));
		_supportedRanges.push_back(Range(rangeId, minValue, maxValue, defaultInterChannelDelay));
	}
	char name[_c::CeboMsrMaxIdentLength];
	_checkError(_c::CeboMsrGetPeripheralName(_handle, _inputIo, name));
	_checkError(_c::CeboMsrGetMinAnalogInputInterChannelDelay(_handle, _inputIo, &_minInterChannelDelay));
	_name = name;
}

int AnalogInput::getDefaultInterChannelDelay(const Range &range) const {
	return range._defaultInterChannelDelay;
}

void AnalogInput::setParameters(const Range &range) const {
	int r;
	_checkError(_c::CeboMsrSetAnalogInputParameters(_handle,_inputIo, range._range, -1, &r));
}

int AnalogInput::setParameters(const Range &range, int interChannelDelay) const {
	int r;
	_checkError(_c::CeboMsrSetAnalogInputParameters(_handle, _inputIo, range._range, interChannelDelay, &r));
	return r;
}

Range AnalogInput::getRange() const {
	_c::CeboMsrRange range;
	int interChannelDelay;
	_checkError(_c::CeboMsrGetAnalogInputParameters(_handle, _inputIo, &range, &interChannelDelay));
	return *std::find(_supportedRanges.begin(), _supportedRanges.end(), Range(range, 0, 0, 0));
}

int AnalogInput::getInterChannelDelay() const {
	_c::CeboMsrRange range;
	int interChannelDelay;
	_checkError(_c::CeboMsrGetAnalogInputParameters(_handle, _inputIo, &range, &interChannelDelay));
	return interChannelDelay;
}

float AnalogInput::read() const {
	float r;
	_checkError(_c::CeboMsrReadAnalogInput(_handle, _inputIo, &r));
	return r;
}

AnalogOutput::AnalogOutput(_c::CeboMsrHandle handle, const _c::CeboMsrIo &io, int index) :
	Output(io), _handle(handle), _index(index) {
	for (int i = 0;; ++i) {
		_c::CeboMsrRange rangeId;
		_checkError(_c::CeboMsrEnumerateRanges(_handle, _outputIo, i, &rangeId));
		if (rangeId == _c::CeboMsrRangeInvalid)
			break;
		float minValue, maxValue;
		_checkError(_c::CeboMsrGetRangeMinMax(_handle, _outputIo, rangeId, &minValue, &maxValue));
		_supportedRanges.push_back(Range(rangeId, minValue, maxValue, 0));
	}
	char name[_c::CeboMsrMaxIdentLength];
	_checkError(_c::CeboMsrGetPeripheralName(_handle, _outputIo, name));
	_name = name;
}

void AnalogOutput::setParameters(const Range &range) const {
	// No supported device actually supports more than the default range ...
}

Range AnalogOutput::getRange() const {
	// No supported device actually supports more than the default range ...
	return _supportedRanges.at(0);
}

void AnalogOutput::write(float value) const {
	_checkError(_c::CeboMsrWriteAnalogOutput(_handle, _outputIo, value));
}

DigitalPort::DigitalPort(_c::CeboMsrHandle handle, const _c::CeboMsrIo &io, int index) :
	Input(io), Output(io), _handle(handle), _index(index), _ioCount(-1) {
	_checkError(_c::CeboMsrGetDigitalPortIoCount(_handle, io, &_ioCount));
	char name[_c::CeboMsrMaxIdentLength];
	_checkError(_c::CeboMsrGetPeripheralName(_handle, _inputIo, name));
	_name = name;
	for (int i = 0; i < _ioCount; ++i) {
		_checkError(_c::CeboMsrGetDigitalPortIoName(_handle, _inputIo, i, name));
		_ioNames.push_back(name);
	}
}

void DigitalPort::setOutputEnableMask(int mask) const {
	_checkError(_c::CeboMsrSetDigitalPortOutputEnableMask(_handle, _inputIo, mask));
}

int DigitalPort::read() const {
	int r;
	_checkError(_c::CeboMsrReadDigitalPort(_handle, _inputIo, &r));
	return r;
}

void DigitalPort::write(int value) const {
	_checkError(_c::CeboMsrWriteDigitalPort(_handle, _outputIo, value));
}

Counter::Counter(_c::CeboMsrHandle handle, const _c::CeboMsrIo &io, int index) :
	Input(io), _handle(handle), _index(index) {
	char name[_c::CeboMsrMaxIdentLength];
	_checkError(_c::CeboMsrGetPeripheralName(_handle, _inputIo, name));
	_name = name;
}

void Counter::reset() const {
	_checkError(_c::CeboMsrResetCounter(_handle, _inputIo));
}

void Counter::setEnabled(bool enable) const {
	_checkError(_c::CeboMsrSetCounterEnable(_handle, _inputIo, enable ? 1 : 0));
}

bool Counter::isEnabled() const {
	int r;
	_checkError(_c::CeboMsrGetCounterEnable(_handle, _inputIo, &r));
	return r != 0;
}

void Counter::setConfig(Counter::CounterConfig counterConfig) const {
	_checkError(_c::CeboMsrSetCounterConfig(_handle, _inputIo, (int)counterConfig));
}

Counter::CounterConfig Counter::getConfig() const {
	int r;
	_checkError(_c::CeboMsrGetCounterConfig(_handle, _inputIo, &r));
	return (CounterConfig)r;
}

int64_t Counter::read() const {
	int64_t r;
	_checkError(_c::CeboMsrReadCounter(_handle, _inputIo, &r));
	return r;
}

Trigger::Trigger(_c::CeboMsrHandle handle, const _c::CeboMsrIo &io, int index) :
	Input(io), _handle(handle), _index(index) {
	char name[_c::CeboMsrMaxIdentLength];
	_checkError(_c::CeboMsrGetPeripheralName(_handle, _inputIo, name));
	_name = name;
}

void Trigger::setEnabled(bool enable) const {
	_checkError(_c::CeboMsrSetTriggerEnable(_handle, _inputIo, enable ? 1 : 0));
}

bool Trigger::isEnabled() const {
	int r;
	_checkError(_c::CeboMsrGetTriggerEnable(_handle, _inputIo, &r));
	return r != 0;
}

void Trigger::setConfig(Trigger::TriggerConfig triggerConfig) const {
	_checkError(_c::CeboMsrSetTriggerConfig(_handle, _inputIo, (int)triggerConfig));
}

Trigger::TriggerConfig Trigger::getConfig() const {
	int r;
	_checkError(_c::CeboMsrGetTriggerConfig(_handle, _inputIo, &r));
	return (TriggerConfig)r;
}

Led::Led(_c::CeboMsrHandle handle, const _c::CeboMsrLed &led, int index) : 
	_handle(handle), _led(led), _index(index) {
	char name[_c::CeboMsrMaxIdentLength];
	_checkError(_c::CeboMsrGetPeripheralName(_handle, _led, name));
	_name = name;
}

void Led::setEnabled(bool enable) const {
	_checkError(_c::CeboMsrSetLed(_handle, _led, enable ? 1 : 0));
}

CurrentSource::CurrentSource(_c::CeboMsrHandle handle, const _c::CeboMsrCurrentSource &source, int index) :
	_handle(handle), _source(source), _index(index) {
	char name[_c::CeboMsrMaxIdentLength];
	_checkError(_c::CeboMsrGetPeripheralName(_handle, _source, name));
	_name = name;
}

float CurrentSource::getReferenceCurrent() const {
	float r;
	_checkError(_c::CeboMsrGetReferenceCurrent(_handle, _source, &r));
	return r;
}

Device::Device(const std::string &ident, DeviceType type, const std::string &firmwareVersion) :
	_ident(ident), _firmwareVersion(firmwareVersion), _type(type), _handle(-1), _maxInputFrames(-1) {
}

void Device::open() {
	_checkError(_c::CeboMsrOpen(const_cast<char*>(_ident.c_str()), &_handle));

	int r;

	_checkError(_c::CeboMsrGetDeviceParameter(_handle, _c::CeboParameterSingleEndedCount, &r));
	_singleEnded.clear();
	for (int i = 0; i < r; ++i)
		_singleEnded.push_back(AnalogInput(_handle, _c::CeboMsrIoIdSingleEnded0 + i, AnalogInput::SingleEnded, i));

	_checkError(_c::CeboMsrGetDeviceParameter(_handle, _c::CeboParameterDifferentialCount, &r));
	_differential.clear();
	for (int i = 0; i < r; ++i)
		_differential.push_back(AnalogInput(_handle, _c::CeboMsrIoIdDifferential0 + i, AnalogInput::Differential, i));

	_checkError(_c::CeboMsrGetDeviceParameter(_handle, _c::CeboParameterAnalogOutputCount, &r));
	_analogOutputs.clear();
	for (int i = 0; i < r; ++i)
		_analogOutputs.push_back(AnalogOutput(_handle, _c::CeboMsrIoIdAnalogOutput0 + i, i));

	_checkError(_c::CeboMsrGetDeviceParameter(_handle, _c::CeboParameterDigitalPortCount, &r));
	_digitalPorts.clear();
	for (int i = 0; i < r; ++i)
		_digitalPorts.push_back(DigitalPort(_handle, _c::CeboMsrIoIdDigitalPort0 + i, i));

	_checkError(_c::CeboMsrGetDeviceParameter(_handle, _c::CeboParameterCounterCount, &r));
	_counters.clear();
	for (int i = 0; i < r; ++i)
		_counters.push_back(Counter(_handle, _c::CeboMsrIoIdCounter0 + i, i));

	_checkError(_c::CeboMsrGetDeviceParameter(_handle, _c::CeboParameterTriggerCount, &r));
	_triggers.clear();
	for (int i = 0; i < r; ++i)
		_triggers.push_back(Trigger(_handle, _c::CeboMsrIoIdTrigger0 + i, i));

	_checkError(_c::CeboMsrGetDeviceParameter(_handle, _c::CeboParameterCurrentSourceCount, &r));
	_currentSources.clear();
	for (int i = 0; i < r; ++i)
		_currentSources.push_back(CurrentSource(_handle, _c::CeboMsrCurrentSource0 + i, i));

	_checkError(_c::CeboMsrGetDeviceParameter(_handle, _c::CeboParameterLedCount, &r));
	_leds.clear();
	for (int i = 0; i < r; ++i)
		_leds.push_back(Led(_handle, _c::CeboMsrLedIndicator + i, 0));

	_checkError(_c::CeboMsrGetDeviceParameter(_handle, _c::CeboParameterMaxInputFrames, &r)); _maxInputFrames = r;
}

void Device::close() {
	_c::CeboMsrClose(_handle);
	_handle = -1;
}

void Device::resetDevice() {
	_checkError(_c::CeboMsrResetDevice(_handle));
}

void Device::resetPeripherals(int mask) {
	_checkError(_c::CeboMsrResetPeripherals(_handle, mask));
}

string Device::getSerialNumber() const {
	char buffer[_c::CeboMsrMaxIdentLength];
	_checkError(_c::CeboMsrGetSerialNumber(_handle, buffer));
	return string(buffer);
}

float Device::getTemperature() const {
	float temp = 0;
	_checkError(_c::CeboMsrGetTemperature(_handle, &temp));
	return temp;
}

void Device::setWatchdogTimeout(unsigned int timeout250ms) {
	_checkError(_c::CeboMsrSetWatchdogTimeout(_handle, timeout250ms));
}

unsigned int Device::getWatchdogTimeout() const {
	unsigned int to;
	_checkError(_c::CeboMsrGetWatchdogTimeout(_handle, &to));
	return to;
}

int Device::calculateMaxBufferedInputFrames() const {
	int r = 0;
	_checkError(_c::CeboMsrCalculateMaxBufferedInputFrames(_handle, &r));
	return r;
}

void Device::setupInputFrame(const InputVector &inputs) {
	vector<_c::CeboMsrIo> ios(inputs.size());
	transform(inputs.begin(), inputs.end(), ios.begin(), input2io());
	_checkError(_c::CeboMsrSetupInputFrame(_handle, &ios[0], (int)ios.size()));
}

float Device::startBufferedDataAcquisition(float frameRate, int frameCount, bool externalStarted) {
	float r = 0;
	if (frameRate <= 0) throw std::invalid_argument("frameRate must be > 0");
	if (frameCount <= 0) throw std::invalid_argument("frameCount must be > 0");
	_checkError(_c::CeboMsrStartDataAcquisition(_handle, externalStarted ? 1 : 0, frameRate, frameCount, &r));
	return r;
}

void Device::startBufferedExternalTimedDataAcquisition(int frameCount) {
	float dummy;
	if (frameCount <= 0) throw std::invalid_argument("frameCount must be > 0");
	_checkError(_c::CeboMsrStartDataAcquisition(_handle, 1, -1, frameCount, &dummy));
}

float Device::startContinuousDataAcquisition(float frameRate, bool externalStarted) {
	float r = 0;
	if (frameRate <= 0) throw std::invalid_argument("frameRate must be > 0");
	_checkError(_c::CeboMsrStartDataAcquisition(_handle, externalStarted ? 1 : 0, frameRate, -1, &r));
	return r;
}

void Device::startContinuousExternalTimedDataAcquisition() {
	float dummy;
	_checkError(_c::CeboMsrStartDataAcquisition(_handle, 1, -1, -1, &dummy));
}

void Device::stopDataAcquisition() {
	_checkError(_c::CeboMsrStopDataAcquisition(_handle));
}

InputFrameVector Device::readBlocking(int frameCount) {
	vector<float> vbuffer(getInputFrameFloatCount() * frameCount);
	vector<int> buffer(getInputFrameIntCount() * frameCount);
	_checkError(_c::CeboMsrReadBlocking(_handle, &vbuffer[0], &buffer[0], frameCount));

	InputFrameVector fb;
	vector<float>::iterator vit = vbuffer.begin();
	vector<int>::iterator it = buffer.begin();
	for (int i = 0; i < frameCount; ++i) {
		fb.push_back(
			InputFrame(
				vit, vit + _singleEnded.size(),
				vit + _singleEnded.size(), vit + _singleEnded.size() + _differential.size(),
				it, it + _digitalPorts.size(),
				it + _digitalPorts.size(), it + _digitalPorts.size() + _counters.size(),
				it + _digitalPorts.size() + _counters.size(), it + _digitalPorts.size() + _counters.size() + _triggers.size()));
		advance(vit, (size_t)getInputFrameFloatCount());
		advance(it, (size_t)getInputFrameIntCount());
	}
	return fb;
}

InputFrameVector Device::readNonBlocking() {
	vector<float> vbuffer(getInputFrameFloatCount() * getMaxInputFramesCount());
	vector<int> buffer(getInputFrameIntCount() * getMaxInputFramesCount());
	int r;
	_checkError(_c::CeboMsrReadNonBlocking(_handle, &vbuffer[0], &buffer[0], &r));

	InputFrameVector fb;
	vector<float>::iterator vit = vbuffer.begin();
	vector<int>::iterator it = buffer.begin();
	for (int i = 0; i < r; ++i) {
		fb.push_back(
			InputFrame(
				vit, vit + _singleEnded.size(),
				vit + _singleEnded.size(), vit + _singleEnded.size() + _differential.size(),
				it, it + _digitalPorts.size(),
				it + _digitalPorts.size(), it + _digitalPorts.size() + _counters.size(),
				it + _digitalPorts.size() + _counters.size(), it + _digitalPorts.size() + _counters.size() + _triggers.size()));
		advance(vit, (size_t)getInputFrameFloatCount());
		advance(it, (size_t)getInputFrameIntCount());
	}
	return fb;
}

InputFrame Device::readFrame() {
	vector<float> vbuffer(getInputFrameFloatCount());
	vector<int> buffer(getInputFrameIntCount());
	_checkError(_c::CeboMsrReadFrame(_handle, &vbuffer[0], &buffer[0]));

	vector<float>::iterator vit = vbuffer.begin();
	vector<int>::iterator it = buffer.begin();
	return
		InputFrame(
			vit, vit + _singleEnded.size(),
			vit + _singleEnded.size(), vit + _singleEnded.size() + _differential.size(),
			it, it + _digitalPorts.size(),
			it + _digitalPorts.size(), it + _digitalPorts.size() + _counters.size(),
			it + _digitalPorts.size() + _counters.size(), it + _digitalPorts.size() + _counters.size() + _triggers.size());
}

void Device::setupOutputFrame(const OutputVector &outputs) {
	vector<_c::CeboMsrIo> ios(outputs.size());
	transform(outputs.begin(), outputs.end(), ios.begin(), output2io());
	_checkError(_c::CeboMsrSetupOutputFrame(_handle, &ios[0], (int)ios.size()));
}

void Device::writeBlocking(const OutputFrameVector &frames) {
	vector<float> vbuffer(getOutputFrameFloatCount() * frames.size());
	vector<int> buffer(getOutputFrameIntCount() * frames.size());
	size_t foffset = 0, ioffset = 0;
	for (OutputFrameVector::const_iterator it = frames.begin(); it != frames.end(); ++it) {
		const OutputFrame &f = *it;
		for (size_t i = 0; i < f._analog.size(); ++i)
			vbuffer[foffset + i] = f._analog[i];
		foffset += f._analog.size();
		for (size_t i = 0; i < f._digital.size(); ++i)
			buffer[ioffset + i] = f._digital[i];
		ioffset += f._digital.size();
	}
	_checkError(_c::CeboMsrWriteBlocking(_handle, &vbuffer[0], &buffer[0], (int)frames.size()));
}

OutputFrameVector Device::writeNonBlocking(const OutputFrameVector &frames) {
	vector<float> vbuffer(getOutputFrameFloatCount() * frames.size());
	vector<int> buffer(getOutputFrameIntCount() * frames.size());
	size_t foffset = 0, ioffset = 0;
	for (OutputFrameVector::const_iterator it = frames.begin(); it != frames.end(); ++it) {
		const OutputFrame &f = *it;
		for (size_t i = 0; i < f._analog.size(); ++i)
			vbuffer[foffset + i] = f._analog[i];
		foffset += f._analog.size();
		for (size_t i = 0; i < f._digital.size(); ++i)
			buffer[ioffset + i] = f._digital[i];
		ioffset += f._digital.size();
	}

	int transferred;
	_checkError(_c::CeboMsrWriteNonBlocking(_handle, &vbuffer[0], &buffer[0], (int)frames.size(), &transferred));

	OutputFrameVector r;
	if ((size_t)transferred < frames.size())
		for (size_t i = (size_t)transferred; i < frames.size(); ++i)
			r.push_back(frames[i]);
	return r;
}

void Device::writeFrame(const OutputFrame &frame) {
	vector<float> vbuffer(frame._analog);
	vector<int> buffer(frame._digital);
	_checkError(_c::CeboMsrWriteFrame(_handle, &vbuffer[0], &buffer[0]));
}

OutputFrameVector Device::createOutputFrames(int count) const {
	OutputFrameVector r;
	for (int i = 0; i < count; ++i)
		r.push_back(OutputFrame(getOutputFrameFloatCount(), getOutputFrameIntCount()));
	return r;
}

string LibraryInterface::getApiVersion() {
	char tmp[_c::CeboMsrMaxIdentLength];
	_c::CeboMsrGetApiVersion(tmp);
	return string(tmp);
}

DeviceVector LibraryInterface::enumerate(const DeviceType &type) {
	int count = -1;

	// Check how many devices can be found ...
	_checkError(_c::CeboMsrBeginEnumerate(&count, type.getInterfaceId()));

	// Read identifier one by one and create instance for return vector.
	DeviceVector r;
	for (int i = 0; i < count; ++i) {
		char ident[_c::CeboMsrMaxIdentLength];
		_c::CeboMsrType type;
		char version[_c::CeboMsrMaxIdentLength];
		_c::CeboMsrGetIdentifier(i, ident, &type, version);

		r.push_back(Device(ident, DeviceType::getTypeById(type), version));
	}
	return r;
}

extern "C" _c::CeboMsrErrorIndex DLLEXPORT CeboMsrBeginEnumerateNetwork(const char *hostname, int port, int *count, _c::CeboMsrType msrType);
DeviceVector LibraryInterface::enumerateNetwork(const char *server, int port, const DeviceType &type) {
	int count = -1;

	// Check how many devices can be found ...
	_checkError(CeboMsrBeginEnumerateNetwork(server, port, &count, type.getInterfaceId()));

	// Read identifier one by one and create instance for return vector.
	DeviceVector r;
	for (int i = 0; i < count; ++i) {
		char ident[_c::CeboMsrMaxIdentLength];
		_c::CeboMsrType type;
		char version[_c::CeboMsrMaxIdentLength];
		_c::CeboMsrGetIdentifier(i, ident, &type, version);

		r.push_back(Device(ident, DeviceType::getTypeById(type), version));
	}
	return r;
}

} // namespace CeboMsr

