#!/usr/bin/env python
# CESYS Software License - Version 1.0 - January 1st, 2016
#
# Permission is hereby granted, free of charge, to any person or organization
# obtaining a copy of the software and accompanying documentation covered by
# this license (the "Software") to use, reproduce, display, distribute,
# execute, and transmit the Software, and to prepare derivative works of the
# Software, and to permit third-parties to whom the Software is furnished to
# do so, all subject to the following:
#
# The software must only be used to operate hardware manufactured by 
# CESYS GmbH, Herzogenaurach.
#
# The copyright notices in the Software and this entire statement, including
# the above license grant, this restriction and the following disclaimer,
# must be included in all copies of the Software, in whole or in part, and
# all derivative works of the Software, unless such copies or derivative
# works are solely in the form of machine-executable object code generated by
# a source language processor.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
# SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
# FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# 
# Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
from CeboMsrApiPython import LibraryInterface, DeviceType

# CeboMsr - Python example - Streaming Input.
#
# Multiple inputs are read multiple times at 300 Hz
# and printed to the console.

def streamingInputTestLC(device):
    """
    Read blocks of data from device.
    device: Device to use for test.
    """
    # Construct selection that contains inputs to read from.
    inputs = [
        device.getSingleEndedInputs()[0],
        device.getSingleEndedInputs()[1],
        device.getDifferentialInputs()[1],
        device.getDigitalPorts()[0],
        device.getDigitalPorts()[1]
    ]

    # Prepare device with this collection ...
    device.setupInputFrame(inputs)

    # Start sampling ...
    device.startBufferedDataAcquisition(300, 20 * 25, False)

    # Read 20 x 25 frames using blocked read,
    # this function returns after *all* (25) requested frames are collected.
    for _ in range(20):
        # Read 25 frames ...
        frames = device.readBlocking(25)

        # Write out the 1st one.
        inFrame = frames[0]
        print(
            ("DigitalPort #0: %d" % inFrame.getDigitalPort(0)) + ", " +
            ("DigitalPort #1: %d" % inFrame.getDigitalPort(1)) + ", " +
            ("SingleEnded #0: %.2f" % inFrame.getSingleEnded(0)) + " V, " +
            ("SingleEnded #1: %.2f" % inFrame.getSingleEnded(1)) + " V, " +
            ("Differential #1: %.2f" % inFrame.getDifferential(1)) + " V")

    # Stop the DAQ.
    device.stopDataAcquisition()

def streamingInputTestStick(device):
    """
    Read blocks of data from device.
    device: Device to use for test.
    """
    # Construct selection that contains inputs to read from.
    inputs = [
        device.getSingleEndedInputs()[0],
        device.getSingleEndedInputs()[1],
        device.getDigitalPorts()[0]
    ]

    # Prepare device with this collection ...
    device.setupInputFrame(inputs)

    # Start sampling ...
    device.startBufferedDataAcquisition(300, 20 * 25, False)

    # Read 20 x 25 frames using blocked read,
    # this function returns after *all* (25) requested frames are collected.
    for _ in range(20):
        # Read 25 frames ...
        frames = device.readBlocking(25)

        # Write out the 1st one.
        inFrame = frames[0]
        print(
            ("DigitalPort #0: %d" % inFrame.getDigitalPort(0)) + ", " +
            ("SingleEnded #0: %.2f" % inFrame.getSingleEnded(0)) + " V, " +
            ("SingleEnded #1: %.2f" % inFrame.getSingleEnded(1)) + " V")

    # Stop the DAQ.
    device.stopDataAcquisition()

def main():
    """
    The examples main method.
    """
    try:
        # Search for devices ...
        devices = LibraryInterface.enumerate(DeviceType.All)

        # If at least one has been found, use the first one ...
        if (len(devices) > 0):
            device = devices[0]

            # Open device, nothing can be done without doing this.
            device.open()

            # Do some streaming ...
            if (device.getDeviceType() == DeviceType.CeboLC):
                streamingInputTestLC(device)
            elif (device.getDeviceType() == DeviceType.CeboStick):
                streamingInputTestStick(device)
            else:
                raise SystemError("No supported device detected.")

            # Finalize device usage, this free's up the device, so it can be used
            # again, including other applications.
            device.close()
    except Exception as e:
        print(e)

if (__name__ == "__main__"):
    main()
