/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/  

package com.cesys.cebo.cebomsr;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Class that is responsible for all device independent functionality.
 *
 * @author Thomas Hoppe
 * @throws IllegalArgumentException If the underlying API detects an illegal argument.
 * @throws IndexOutOfBoundsException If the underlying API was forced to access an invalid array element.
 * @throws IllegalStateException If a method is called that should not be called in the current state.
 * @throws IOException Unrecoverable error in the lower API levels, in most cases an unpredictable communication error with the hardware.
 * @throws RuntimeException If the API reports an unknown error, this should never be occur except for the case of a bug inside the API.
 */
public final class LibraryInterface {
	private LibraryInterface(){}


	/**
	 * Request version of the CeboMsr-Base-API.
	 */
	public static String getApiVersion() throws IOException {
		byte[] tmp = new byte[SystemConnector.MaxIdentLength];
		SystemConnector.checkError(SystemConnector.CeboMsrGetApiVersion(tmp));
		return Utils.convertByteArrayToString(tmp);
	}

	/**
	 * Enumerate devices on this system. Already opened devices are not included.
	 */
	public static List<Device> enumerate(DeviceType type) throws IOException {
		ArrayList<Device> r = new ArrayList<Device>();

		// Get count of devices.
		int[] countb = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrBeginEnumerate(countb, type.getInterfaceId()));
		int count = countb[0];

		// Get info about all devices.
		byte[] ident = new byte[SystemConnector.MaxIdentLength];
		int[] msrType = new int[1];
		byte[] version = new byte[SystemConnector.MaxIdentLength];
		for (int i = 0; i < count; ++i) {
			// Get info ...
			SystemConnector.checkError(SystemConnector.CeboMsrGetIdentifier(i, ident, msrType, version));

			// Add device.
			r.add(new Device(
					Utils.convertByteArrayToString(ident),
					DeviceType.getDeviceTypeById(msrType[0]),
					Utils.convertByteArrayToString(version)));
		}

		return Collections.unmodifiableList(r);
	}

	/**
	 * *** unofficial experimental feature, don't use ***
	 */
	public static List<Device> enumerateNetwork(String hostname, int port, DeviceType type) throws IOException {
		ArrayList<Device> r = new ArrayList<Device>();

		// Get count of devices.
		int[] countb = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrBeginEnumerateNetwork(hostname, port, countb, type.getInterfaceId()));
		int count = countb[0];

		// Get info about all devices.
		byte[] ident = new byte[SystemConnector.MaxIdentLength];
		int[] msrType = new int[1];
		byte[] version = new byte[SystemConnector.MaxIdentLength];
		for (int i = 0; i < count; ++i) {
			// Get info ...
			SystemConnector.checkError(SystemConnector.CeboMsrGetIdentifier(i, ident, msrType, version));

			// Add device.
			r.add(new Device(
					Utils.convertByteArrayToString(ident),
					DeviceType.getDeviceTypeById(msrType[0]),
					Utils.convertByteArrayToString(version)));
		}

		return Collections.unmodifiableList(r);
	}
}
