/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/  

package com.cesys.cebo.cebomsr.test;

import java.io.IOException;
import java.util.List;

import com.cesys.cebo.cebomsr.*;

public class Example {

	/**
	 * Different examples showing communication using data blocks.
	 * @throws IOException
	 */
	void streamingIoTest(Device device) throws IOException {
		// Construct selection that contains inputs to read from.
		Input[] inputs = {
			device.getSingleEndedInputs().get(0),
			device.getSingleEndedInputs().get(1),
			device.getDifferentialInputs().get(0),
			device.getDigitalPorts().get(0),
			device.getDigitalPorts().get(1)
		};

		// Prepare device with this collection ...
		device.setupInputFrame(inputs);

		// Start sampling ...
		//device.startDataAcquisition(false, 300, 20 * 25);

		// Read 20 x 25 frames using blocked read, this function returns after *all* requested frames are collected.
		for (int i = 0; i < 20; ++i) {
			// Read 25 frames ...
			List<InputFrame> frames = device.readBlocking(25);

			// Write out the 1st one.
			InputFrame inFrame = frames.get(0);
			System.out.printf(
					"DigitalPort #0: 0x%02x, DigitalPort #1: 0x%02x, SingleEnded #0: %.2f V, " +
					"SingleEnded #1: %.2f V, Differential #1: %.2f V\n",
					inFrame.getDigitalPort(0), inFrame.getDigitalPort(1), inFrame.getSingleEnded(0),
					inFrame.getSingleEnded(1), inFrame.getDifferential(1));
		}
	}

	/**
	 * Do frame IO (read/write multiple I/O in single calls).
	 * @throws IOException
	 */
	void frameIoTest(Device device) throws IOException {
		// Prepare and fill list of inputs to read. The order of the inputs is irrelevant,
		// but no input can be specified more than once.
		Input[] inputs = {
			device.getSingleEndedInputs().get(0),
			device.getSingleEndedInputs().get(1),
			device.getDifferentialInputs().get(1),
			device.getDigitalPorts().get(0),
			device.getDigitalPorts().get(1),
			device.getCounters().get(0)
		};

		// Prepare device with this selection.
		device.setupInputFrame(inputs);

		// Read the values multiple times and write them to the console.
		for (int i = 0; i < 100; ++i) {
			// Read all inputs into the instance of InputFrame.
			InputFrame inFrame = device.readFrame();

			// Write results to the console.
			System.out.printf(
					"DigitalPort #0: 0x%02x, DigitalPort #1: 0x%02x, SingleEnded #0: %.2f V, " +
					"SingleEnded #1: %.2f V, Differential #1: %.2f V, Counter #0: %d\n",
					inFrame.getDigitalPort(0), inFrame.getDigitalPort(1), inFrame.getSingleEnded(0),
					inFrame.getSingleEnded(1), inFrame.getDifferential(1), inFrame.getCounter(0));
		}

		// Write to analog out #1 and digital out #2 in one call.
		// Prepare and set output selection.
		Output[] outputs = {
			device.getAnalogOutputs().get(1),
			device.getDigitalPorts().get(2)
		};

		// Prepare device ...
		device.setupOutputFrame(outputs);

		// Prepare digital port #2 as output.
		device.getDigitalPorts().get(2).setOutputEnableMask(0xf);

		// Create instance of OutputFrame. There's no direct construction, as this
		// instance may vary between different device types.
		OutputFrame outFrame = device.createOutputFrame();

		// Write it to hardware, modify contents in every looping.
		for (int i = 0; i < 100; ++i) {
			outFrame.setAnalogOutput(1, (float)(3 * Math.sin((float)i / 100.f)));
			outFrame.setDigitalPort(2, i);

			device.writeFrame(outFrame);
		}

		// Reset to power up defaults.
		device.resetDevice();
	}

	/**
	 * Do very simple IO, digital as well as analog.
	 * @throws IOException
	 */
	void directIoTest(Device device) throws IOException {
//		// Read digital port #0 and write result to digital port #1 (mirror pins).
//		// Configure port first, all bits of digital port #0 to in, and all bits of digital port #1 to out.
//		DigitalPort p0 = device.getDigitalPorts().get(0);
//		DigitalPort p1 = device.getDigitalPorts().get(1);
//		p1.setOutputEnableMask(0xff);
//
//		// Read from #0 ...
//		int value = p0.read();
//
//		// Write the inverse value ...
//		p1.write(~value);
//
//		// Set gain to 1 (+- 10V) on single ended analog input #0.
//		device.getSingleEndedInputs().get(0).setParameters(Gain.Gain1);
//
//		// Read single ended #0 in voltage ...
//		float vvalue = device.getSingleEndedInputs().get(0).read();
//
//		// Write value to analog output #1
//		device.getAnalogOutputs().get(1).write(vvalue);
//
//		device.getAnalogOutputs().get(0).write(device.getSingleEndedInputs().get(1).read());
//
//		// Reset to power up defaults.
//		device.resetDevice();
	}

	/**
	 * Print various information to the console.
	 * @throws IOException
	 */
	void printInformation(Device device) throws IOException {
		// Put out some device specific information.
		System.out.println("API version: " + LibraryInterface.getApiVersion());
		System.out.println("Firmware version: " + device.getFirmwareVersion());
		System.out.println("Identifier: " + device.getIdentifier());
		System.out.println("Serial number: " + device.getSerialNumber());
		System.out.println("Temperature: " + device.getTemperature());
	}

	/**
	 * Called by main().
	 */
	public void runExamples() {
		// Nearly all methods can raise exceptions. For this example, the error handling is done at this single point.
		try {
			// Search for devices ...
			List<Device> devices = LibraryInterface.enumerate(DeviceType.All);

			// If at least one has been found, use the first one ...
			if (!devices.isEmpty()) {
				Device device = devices.get(0);

				// Open device, nothing can be done without doing this.
				device.open();

				//float x = device.getSingleEndedInputs().get(0).read();

				// Finalize device usage, this free's up the device, so it can be used
				// again, including other applications.
				device.close();
			}
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Application entry point.
	 * @param args
	 */
	public static void main(String[] args) {
		new Example().runExamples();
	}
}
